(in-package :crypto-tests)

(defun ascii-string-to-byte-array (string)
  (let ((vec (make-array (length string) :element-type '(unsigned-byte 8))))
    (dotimes (i (length string) vec)
      (let ((byte (char-code (char string i))))
        (assert (< byte 256))
        (setf (aref vec i) byte)))))

(defun hex-string-to-byte-array (string &key (start 0) (end nil))
  (declare (type string string))
  (let* ((end (or end (length string)))
         (length (/ (- end start) 2))
         (key (make-array length :element-type '(unsigned-byte 8))))
    (declare (type (simple-array (unsigned-byte 8) (*)) key))
    (flet ((char-to-digit (char)
             (let ((x (position char "0123456789abcdef" :test #'char-equal)))
               (or x (error "Invalid hex key ~A specified" string)))))
      (loop for i from 0
            for j from start below end by 2
            do (setf (aref key i)
                     (+ (* (char-to-digit (char string j)) 16)
                        (char-to-digit (char string (1+ j)))))
            finally (return key)))))

(defmacro ecb-mode-test (hexkey hexinput hexoutput)
  (let* ((cipher (symbol-value '*test-cipher*))
         (e-test-name (intern (format nil "~A.ENCRYPT.~D" cipher
                                      (symbol-value '*test-number*))))
         (d-test-name (intern (format nil "~A.DECRYPT.~D" cipher
                                      (symbol-value '*test-number*)))))
    `(progn
      (eval-when (:compile-toplevel)
        (incf *test-number*))
      (rt:deftest ,e-test-name
          (ecb-mode-test-guts ,cipher #'crypto:encrypt
           ,hexkey ,hexinput ,hexoutput)
        t)
      (rt:deftest ,d-test-name
        (ecb-mode-test-guts ,cipher #'crypto:decrypt
         ,hexkey ,hexoutput ,hexinput)
        t))))

(defun frob-hex-string (cipher-name func hexkey hexinput)
  (let* ((key (hex-string-to-byte-array hexkey))
         (input (hex-string-to-byte-array hexinput))
         (cipher (crypto:make-cipher cipher-name :ecb key))
         (scratch (copy-seq input)))
    (funcall func cipher input scratch)
    scratch))

(defun ecb-mode-test-guts (cipher-name func hexkey hexinput hexoutput)
  (let* ((output (hex-string-to-byte-array hexoutput)))
    (not (mismatch (frob-hex-string cipher-name func hexkey hexinput)
                   output))))

(defmacro digest-test (string hexdigest)
  (let* ((digest (symbol-value '*test-digest*))
         (test-name (intern (format nil "~A-STRING.~D" digest
                                    (symbol-value '*test-number*)))))
    `(progn
      (eval-when (:compile-toplevel)
        (incf *test-number*))
      (rt:deftest ,test-name (digest-test-guts ,digest ,string ,hexdigest) t))))

(defmacro digest-bit-test (leading byte trailing hexdigest)
  (let* ((digest (symbol-value '*test-digest*))
         (test-name (intern (format nil "~A-BIT.~D" digest
                                    (symbol-value '*test-number*)))))
    `(progn
      (eval-when (:compile-toplevel)
        (incf *test-number*))
      (rt:deftest ,test-name (digest-bit-test-guts ,digest ,leading ,byte ,trailing ,hexdigest) t))))

(defun digest-test-guts (digest string hexdigest)
  (let* ((input (ascii-string-to-byte-array string))
         (expected-digest (hex-string-to-byte-array hexdigest))
         (result (crypto:digest-sequence digest input)))
    (not (mismatch result expected-digest))))

(defun digest-bit-test-guts (digest leading byte trailing hexdigest)
  (let* ((input (let ((vector (make-array (+ 1 leading trailing)
                                          :element-type '(unsigned-byte 8)
                                          :initial-element 0)))
                  (setf (aref vector leading) byte)
                  vector))
         (expected-digest (hex-string-to-byte-array hexdigest))
         (result (crypto:digest-sequence digest input)))
    (not (mismatch result expected-digest))))
